#pragma once

#include <string>
#include <vector>
#include <thread>
#include <mutex>

#include "readerwriterqueue.h"

struct AVFormatContext;
struct AVCodecContext;
struct AVFrame;
struct AVFilterContext;
struct AVFilterGraph;
struct AVPacket;
struct AVAudioFifo;

class LibavStreamer
{
public:

	LibavStreamer();
	~LibavStreamer();

	bool OpenMic(std::wstring micName);
	bool OpenCam(std::wstring camName);
	bool OpenCamMic(std::wstring camName, std::wstring micName);

	bool OpenOutputUrl(std::wstring url);

	bool InitFilters();
	bool InitFifo();

	bool StartDecodingThread();
	bool StopDecodingThread();

	bool StartVideoEncodingThread();
	bool StopVideoEncodingThread();

	bool StartAudioEncodingThread();
	bool StopAudioEncodingThread();

	void FlushFiltersEncoders();

	void Cleanup();

	bool Volume(double volume);

	bool Camera(bool attached);

private:

	std::string ConvertFromWString(std::wstring wstr);

	bool OpenDShow(std::wstring dshowInputString);

	bool InitFilter(AVCodecContext* pDecodingContext, AVCodecContext* pEncodingContext, const char* pFilterSpec);

	bool FilterEncodeWriteFrame(AVFrame* frame, unsigned int stream_index);

	bool EncodeWriteFrame(unsigned int stream_index, bool flush);

	bool FilterEncodeWriteFrameAudio(AVFrame* pFrame, unsigned int streamIndex, AVAudioFifo* pFifo);

	bool AddSamplesToFifo(AVAudioFifo* pFifo, uint8_t** convertedInputSamples, const int frameSize);

	bool LoadEncodeWrite(AVAudioFifo* pFifo,
		AVFormatContext* pOutputFormatContext,
		AVCodecContext* pOutputCodecContext,
		unsigned int streamIndex, 
		int64_t& framePts);

	bool EncodeWriteAudioFrame(unsigned int streamIndex, AVFrame* pFrame);

private:

	AVFormatContext* m_pInputFormatContext = { nullptr };

	typedef struct StreamContext
	{
		AVCodecContext* pDecodingContext = { nullptr };
		AVCodecContext* pEncodingContext = { nullptr };

		moodycamel::ReaderWriterQueue<AVPacket*>* pPacketsQueue = { nullptr };
	};

	std::vector<StreamContext> m_streamContexts;

	AVFormatContext* m_pOutputFormatContext = { nullptr };

	typedef struct FilteringContext
	{
		AVFilterContext* pBufferSinkContext = { nullptr };
		AVFilterContext* pBufferSourceContext = { nullptr };
		AVFilterGraph* pFilterGraph = { nullptr };

		AVPacket* pEncodedPacket = { nullptr };
		AVFrame* pFilteredFrame = { nullptr };
	};

	std::vector<FilteringContext> m_filteringContexts;

	AVAudioFifo* pFifo = { nullptr };

	AVPacket* pReceivingPacket = { nullptr };

	bool m_packetReceiverThreadStarted = { false };
	bool m_videoEncodingThreadStarted = { false };
	bool m_audioEncodingThreadStarted = { false };

	std::unique_ptr<std::thread> m_pPacketReceiverThread;
	std::unique_ptr<std::thread> m_pVideoEncodingThread;
	std::unique_ptr<std::thread> m_pAudioEncodingThread;

	std::mutex m_writeFrameMutex;


private:

	static int WriteFrame(LibavStreamer* pLibavStreamer, AVFormatContext* pOutputFormatContext, AVPacket* pPacket);

	static void PacketReceiverThreadProc(LibavStreamer* pLibavStreamer);
	static void VideoDecodingEncodingThreadProc(LibavStreamer* pLibavStreamer, int streamIndex);
	static void AudioDecodingEncodingThreadProc(LibavStreamer* pLibavStreamer, int streamIndex);

	//static bool m_isCameraAttached;
};

