// MediaBroadcast.cpp : Defines the exported functions for the DLL.
//

#include "framework.h"
#include "MediaBroadcast.h"
#include "LibavStreamer.h"


// This is an example of an exported variable
//MEDIABROADCAST_API int nMediaBroadcast=0;

// This is an example of an exported function.
//MEDIABROADCAST_API int fnMediaBroadcast(void)
//{
//    return 0;
//}

// This is the constructor of a class that has been exported.
CMediaBroadcast::CMediaBroadcast()
{
    m_pLibavStreamer = new LibavStreamer();
}

CMediaBroadcast::~CMediaBroadcast()
{
    LibavStreamer* pLibavStreamer = static_cast<LibavStreamer*>(m_pLibavStreamer);
    delete pLibavStreamer;
    m_pLibavStreamer = nullptr;
}

bool CMediaBroadcast::StartStreamingViaRTMP(const wchar_t* micName, const wchar_t* url)
{
    LibavStreamer* pLibavStreamer = static_cast<LibavStreamer*>(m_pLibavStreamer);

    if (!pLibavStreamer->OpenMic(std::wstring(micName)))
    {
        return false;
    }

    if (!pLibavStreamer->OpenOutputUrl(std::wstring(url)))
    {
        pLibavStreamer->Cleanup();
        return false;
    }

    if (!pLibavStreamer->InitFilters())
    {
        pLibavStreamer->Cleanup();
        return false;
    }

    if (!pLibavStreamer->InitFifo())
    {
        pLibavStreamer->Cleanup();
        return false;
    }

    if (!pLibavStreamer->StartAudioEncodingThread())
    {
        pLibavStreamer->Cleanup();
        return false;
    }

    if (!pLibavStreamer->StartDecodingThread())
    {
        pLibavStreamer->StopAudioEncodingThread();
        pLibavStreamer->Cleanup();
        return false;
    }

    return true;
}

bool CMediaBroadcast::StartStreamingViaRTMP(const wchar_t* camName, const  wchar_t* micName, const wchar_t* url)
{
    LibavStreamer* pLibavStreamer = static_cast<LibavStreamer*>(m_pLibavStreamer);

    if (!pLibavStreamer->OpenCamMic(std::wstring(camName), std::wstring(micName)))
    {
        return false;
    }

    if (!pLibavStreamer->OpenOutputUrl(std::wstring(url)))
    {
        pLibavStreamer->Cleanup();
        return false;
    }

    if (!pLibavStreamer->InitFilters())
    {
        pLibavStreamer->Cleanup();
        return false;
    }

    if (!pLibavStreamer->InitFifo())
    {
        pLibavStreamer->Cleanup();
        return false;
    }

    if (!pLibavStreamer->StartVideoEncodingThread())
    {
        pLibavStreamer->Cleanup();
        return false;
    }

    if (!pLibavStreamer->StartAudioEncodingThread())
    {
        pLibavStreamer->StopVideoEncodingThread();
        pLibavStreamer->Cleanup();
        return false;
    }

    if (!pLibavStreamer->StartDecodingThread())
    {
        pLibavStreamer->StopAudioEncodingThread();
        pLibavStreamer->StopVideoEncodingThread();
        pLibavStreamer->Cleanup();
        return false;
    }

    return true;
}

void CMediaBroadcast::StopStreaming()
{
    LibavStreamer* pLibavStreamer = static_cast<LibavStreamer*>(m_pLibavStreamer);

    pLibavStreamer->StopDecodingThread();
    pLibavStreamer->StopAudioEncodingThread();
    pLibavStreamer->StopVideoEncodingThread();

    pLibavStreamer->Cleanup();
}

bool CMediaBroadcast::Volume(double volume)
{
    LibavStreamer* pLibavStreamer = static_cast<LibavStreamer*>(m_pLibavStreamer);

    return pLibavStreamer->Volume(volume);
}

bool CMediaBroadcast::Camera(bool attached)
{
    LibavStreamer* pLibavStreamer = static_cast<LibavStreamer*>(m_pLibavStreamer);

    return pLibavStreamer->Camera(attached);
}